/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.List;

import nan.tools.EventLogger;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * A representation of an NNTP message in RFC 850 format.
 *
 * @author John David Ratliff
 * @version 1.9, 1/6/04
 */
public class Message implements XMLSerializable, Transposer, Selected {
    private MessageHeader header;
    private String body;
    private List attachments = new ArrayList();

    /**
     * XML tag for serialization purposes.
     */
    public static final String XML_TAG = "message";

    /**
     * XML tag for the body property.
     */
    public static final String XML_BODY = "body";

    /**
     * Creates a new Message object.
     *
     * @param header The MessageHeader of this Message.
     * @param body The body of this message.
     */
    public Message(MessageHeader header, String body) {
        this.header = header;

        if (body == null) {
            this.body = "";
        } else {
            this.body = body;
        }
    }

    /**
     * Creates a new Message object.
     */
    public Message() {
    }

    /**
     * Gets the quoted message body for use in replying.
     *
     * @return The quoted message String.
     */
    public String getQuotedBody() {
        StringBuffer buffer = new StringBuffer();

        buffer.append('\n');
        buffer.append('\n');

        try {
            BufferedReader reader = new BufferedReader(new StringReader(getBody()));
            String line;

            while ((line = reader.readLine()) != null) {
                buffer.append('>');
                buffer.append(' ');
                buffer.append(line);
                buffer.append('\n');
            }
        } catch (IOException e) {
            // this seems unlikely to occur...
            EventLogger.getInstance().logError("IOException during Message.getQuotedBody()...");
        }

        return buffer.toString();
    }

    /**
     * Gets the body of this Message.
     *
     * @return The body.
     */
    public String getBody() {
        return this.body;
    }

    /**
     * Sets the body of this Message.
     *
     * @param body The body.
     */
    public void setBody(String body) {
        this.body = body;
    }

    /**
     * Gets the header of this Message.
     *
     * @return The MessageHeader.
     */
    public MessageHeader getHeader() {
        return this.header;
    }

    /**
     * Sets the header of this Message.
     *
     * @param header The MessageHeader.
     */
    public void setHeader(MessageHeader header) {
        this.header = header;
    }

    /**
     * Asks if this message has been read.
     *
     * @return true if this message has been read; false otherwise.
     */
    public boolean isRead() {
        return header.isRead();
    }

    /**
     * Sets if this message has been read.
     *
     * @param read true if read; false otherwise.
     */
    public void setRead(boolean read) {
        header.setRead(read);
    }

    /**
     * Gets the count of attachments on this message.
     *
     * @return The Attachment count.
     */
    public int getAttachmentCount() {
        return attachments.size();
    }

    /**
     * Adds an attachment to this Message.
     *
     * @param attachment The Attachment.
     */
    public void addAttachment(Attachment attachment) {
        attachments.add(attachment);
    }

    /**
     * Removes an attachment from this Message.
     *
     * @param index The index of the attachment.
     *
     * @return The removed Attachment.
     *
     * @throws IndexOutOfBoundsException if index is not valid (0 &lt;=
     *         index &lt; size).
     */
    public Attachment removeAttachment(int index)
                                throws IndexOutOfBoundsException {
        return (Attachment)attachments.remove(index);
    }

    /**
     * Gets an Attachment from this message.
     *
     * @param index The index number of the Attachment.
     *
     * @return The Attachment.
     *
     * @throws IndexOutOfBoundsException if the index is invalid (0 &lt;
     *         index &lt; <code>getAttachmentCount()</code>).
     */
    public Attachment getAttachment(int index)
                             throws IndexOutOfBoundsException {
        if ((index < 0) || (index >= attachments.size())) {
            throw new IndexOutOfBoundsException();
        }

        return (Attachment)attachments.get(index);
    }

    /**
     * Converts an object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // the root element
        Element root = document.addElement(XML_TAG);

        // the headers
        Document headerDocument = this.header.toXML();
        Element headerRoot = headerDocument.getRootElement();
        headerRoot.setParent(null);
        root.add(headerRoot);

        // the body
        Element body = root.addElement(XML_BODY);
        body.setText(getBody());

        // the attachments
        for (int i = 0; i < getAttachmentCount(); i++) {
            Attachment attachment = getAttachment(i);
            Element xmlAttachment = attachment.toXML().getRootElement();
            xmlAttachment.setParent(null);
            root.add(xmlAttachment);
        }

        return document;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof Message) {
            Message message = (Message)object;

            this.header = message.header;
            this.body = message.body;
            this.attachments = message.attachments;
        }
    }

    /**
     * Returns a String representation of this Message.
     *
     * @return A String representation.
     */
    public String toString() {
        return header.toString();
    }
}
