/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.Date;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import com.sun.treetable.TreeTableModel;

import nan.tools.EventLogger;

/**
 * An implementation of the TreeTableModel interface for JTreeTable.
 *
 * @author John David Ratliff
 * @version 1.3, 1/3/04
 */
public class MessageTreeTableModel extends DefaultTreeModel
    implements TreeTableModel {
    private static final String EMPTY = "";
    private static final String[] COLUMN_NAMES = {"From", "Subject", "Date"};
    private static final Class[] CLASSES = {
                                               TreeTableModel.class,
                                               String.class, Date.class
                                           };

    /**
     * Creates a new MessageTreeTableModel object.
     */
    public MessageTreeTableModel() {
        super(new DefaultMutableTreeNode());
    }

    /**
     * Gets the class for the particular column requested.
     *
     * @param column The column number.
     *
     * @return The type of object in that column.
     */
    public Class getColumnClass(int column) {
        return CLASSES[column];
    }

    /**
     * Gets the column count.
     *
     * @return The column count.
     */
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }

    /**
     * Gets the name of a column.
     *
     * @param column The column number.
     *
     * @return The column name.
     */
    public String getColumnName(int column) {
        return COLUMN_NAMES[column];
    }

    /**
     * Checks if a cell is editable in the table.
     *
     * @param node The node being tested.
     * @param column The column number.
     *
     * @return true if the cell is editable; false otherwise.
     */
    public boolean isCellEditable(Object node, int column) {
        return getColumnClass(column) == TreeTableModel.class;
    }

    /**
     * Sets the value of a cell. This method does nothing.
     *
     * @param value ignored.
     * @param node ignored.
     * @param column ignored.
     */
    public void setValueAt(Object value, Object node, int column) {
    }

    /**
     * Gets the value of a cell.
     *
     * @param node The tree node.
     * @param column The column number.
     *
     * @return The cell value.
     */
    public Object getValueAt(Object node, int column) {
        DefaultMutableTreeNode mtNode = (DefaultMutableTreeNode)node;
        Object object = mtNode.getUserObject();

        if (object == null) {
            return EMPTY;
        }

        MessageHeader header = null;

        if (object instanceof MessageHeader) {
            header = (MessageHeader)object;
        } else if (object instanceof Message) {
            header = ((Message)object).getHeader();
        } else {
            EventLogger.getInstance().logError("TreeTable object is neither Message nor " +
                                               "MessageHeader in " +
                                               "MessageTreeTableModel.getValueAt()...");
        }

        return header;
    }

    /**
     * Returns a String representation of this model.
     *
     * @return A String representation.
     */
    public String toString() {
        return "MessageTreeTableModel";
    }
}
