/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.io.File;

import java.lang.reflect.Method;

import nan.NaN;

import nan.exception.XMLParseException;

import nan.ssh.SSHConnection;

import nan.tools.BackgroundReader;
import nan.tools.SettingsArchiver;
import nan.tools.XMLSerializer;

import nan.view.PasswordDialog;
import nan.view.WaitDialog;

import org.dom4j.Document;

/**
 * The collective Settings for the application.
 *
 * @author John David Ratliff
 * @version 1.13, 1/4/04
 */
public class Settings {
    private UserSettings userSettings;
    private ServerSettings serverSettings;
    private NewsgroupList availableNewsgroups;
    private NewsgroupList subscribedNewsgroups;
    private BackgroundReader reader;
    private boolean userSettingsLoaded;
    private boolean serverSettingsLoaded;

    /**
     * Creates a new Settings object.
     */
    public Settings() {
        this.userSettings = new UserSettings();
        this.serverSettings = new ServerSettings();
        this.availableNewsgroups = new NewsgroupList();
        this.subscribedNewsgroups = new NewsgroupList();

        ServerSettings server = loadServerSettings();

        if (server != null) {
            serverSettings.transpose(server);
            serverSettingsLoaded = true;

            if (serverSettings.useRemoteSave()) {
                // remote load
                PasswordDialog dialog = new PasswordDialog(null,
                                                           serverSettings);
                dialog.display();

                // create the SSH connection
                SSHConnection.getInstance().connect(serverSettings);

                SSHConnection.getInstance().download(serverSettings.getDirectory(),
                                                     NaN.ARCHIVE_FILE.getName());

                if (NaN.ARCHIVE_FILE.exists()) {
                    SettingsArchiver.unarchive();
                    NaN.ARCHIVE_FILE.delete();
                } else {
                    // unable to load settings...
                    serverSettings.setDirectory(null);
                }
            }
        }

        reloadSettings();
    }

    /**
     * Reloads the settings from cache.
     */
    public void reloadSettings() {
        UserSettings user = loadUserSettings();

        if (user != null) {
            userSettings.transpose(user);
            userSettingsLoaded = true;
        }

        File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                              NaN.AVAILABLE_NEWSGROUPS_FILE);
        NewsgroupList available = loadNewsgroupsList(file, false);

        file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                         NaN.SUBSCRIBED_NEWSGROUPS_FILE);

        NewsgroupList subscribed = loadNewsgroupsList(file, true);

        if (subscribed != null) {
            this.subscribedNewsgroups.transpose(subscribed);
        }
    }

    /**
     * Asks if the user settings were loaded.
     *
     * @return true if the user settings were loaded from cache; false
     *         otherwise.
     */
    public boolean loadedUserSettings() {
        return this.userSettingsLoaded;
    }

    /**
     * Loads the UserSettings from file.
     *
     * @return The UserSettings or null if they could not be loaded.
     */
    private UserSettings loadUserSettings() {
        UserSettings settings = null;

        try {
            File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                                  NaN.USER_SETTINGS_FILE);
            Document document = XMLSerializer.readDocument(file);
            settings = XMLFactory.getUserSettings(document);
        } catch (XMLParseException e) {
            // unable to read user settings from file
            // these errors are logged elsewhere
        }

        return settings;
    }

    /**
     * Asks if ths server settings were loaded.
     *
     * @return true if they were loaded; false otherwise.
     */
    public boolean loadedServerSettings() {
        return this.serverSettingsLoaded;
    }

    /**
     * Loads the server settings from file.
     *
     * @return The loaded ServerSettings or null if they could not be
     *         loaded.
     */
    private ServerSettings loadServerSettings() {
        ServerSettings settings = null;

        try {
            File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                                  NaN.SERVER_SETTINGS_FILE);
            Document document = XMLSerializer.readDocument(file);
            settings = XMLFactory.getServerSettings(document);
        } catch (XMLParseException e) {
            // unable to read server settings from file
            // these errors are logged elsewhere
        }

        return settings;
    }

    /**
     * Loads a NewsgroupList from file.
     *
     * @param file The file of the NewsgroupList to load.
     * @param wait If the method should block until it's finished or not.
     *
     * @return The loaded NewsgroupList.
     */
    private NewsgroupList loadNewsgroupsList(File file, boolean wait) {
        NewsgroupList list;

        if (wait) {
            list = new NewsgroupList();
        } else {
            list = this.availableNewsgroups;
        }

        BackgroundReader reader = null;

        try {
            Class[] argTypes = {Document.class};
            Method method = XMLFactory.class.getMethod("getNewsgroupList",
                                                       argTypes);
            reader = new BackgroundReader(list, file, method);
            reader.start();
        } catch (NoSuchMethodException e) {
            // this will NEVER happen, unless someone changes it...
            System.out.println("Don't change the method names in XMLFactory!");

            e.printStackTrace();
            System.exit(-1);
        }

        if (wait) {
            while (!reader.isFinished()) {
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                    // this isn't a problem...
                }
            }
        } else {
            this.reader = reader;
        }

        return list;
    }

    /**
     * Gets the UserSettings.
     *
     * @return The UserSettings.
     */
    public UserSettings getUserSettings() {
        return this.userSettings;
    }

    /**
     * Gets the ServerSettings.
     *
     * @return The ServerSettings.
     */
    public ServerSettings getServerSettings() {
        return this.serverSettings;
    }

    /**
     * Asks if the available newsgroup list is ready.
     *
     * @return true if it is ready; false otherwise.
     */
    public boolean isAvailableNewsgroupsReady() {
        return reader.isFinished();
    }

    /**
     * Sets the wait dialog for the available background task.
     *
     * @param dialog The WaitDialog sleeper.
     */
    public void setWaitDialog(WaitDialog dialog) {
        reader.setWaitDialog(dialog);
    }

    /**
     * Gets the available newsgroups.
     *
     * @return The available newsgroups.
     */
    public NewsgroupList getAvailableNewsgroups() {
        return this.availableNewsgroups;
    }

    /**
     * Gets the subscribed newsgroups.
     *
     * @return The subscribed newsgroups.
     */
    public NewsgroupList getSubscribedNewsgroups() {
        return this.subscribedNewsgroups;
    }

    /**
     * Returns a String representation of these Settings.
     *
     * @return A String representation.
     */
    public String toString() {
        return "Settings[user settings=" + getUserSettings() +
               ", server settings=" + getServerSettings() +
               ", available newsgroups=" + getAvailableNewsgroups() +
               ", subscribed newsgroups=" + getSubscribedNewsgroups() + "]";
    }
}
