/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * The settings used by the user.
 *
 * @author John David Ratliff
 * @version 1.5, 1/2/04
 */
public class UserSettings implements XMLSerializable, Transposer {
    private String name;
    private String email;
    private String signature;

    /**
     * The XML tag for serializing user settings.
     */
    public static final String XML_TAG = "user-settings";

    /**
     * XML tag for the name setting.
     */
    public static final String XML_NAME = "name";

    /**
     * XML tag for the email setting.
     */
    public static final String XML_EMAIL = "email";

    /**
     * XML tag for the signature setting.
     */
    public static final String XML_SIGNATURE = "signature";

    /**
     * Creates a new UserSettings object.
     *
     * @param name The name of the user.
     * @param email The email of the user.
     * @param signature The signature of the user.
     */
    public UserSettings(String name, String email, String signature) {
        this.name = name;
        this.email = email;
        this.signature = signature;
    }

    /**
     * Creates a new UserSettings object.
     */
    public UserSettings() {
        this.name = "John Doe";
        this.email = "johndoe@hotmail.com";
        this.signature = "";
    }

    /**
     * Converts this UserSettings object into an XML Document.
     *
     * @return The XML Document.
     */
    public Document toXML() {
        Document document = DocumentHelper.createDocument();

        // the root element
        Element root = document.addElement(XML_TAG);

        // the settings
        Element name = root.addElement(XML_NAME);
        name.setText(getName());

        Element email = root.addElement(XML_EMAIL);
        email.setText(getEmail());

        Element signature = root.addElement(XML_SIGNATURE);
        signature.setText(getSignature());

        return document;
    }

    /**
     * Gets the user's name.
     *
     * @return The name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the user's name.
     *
     * @param name The name.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Gets the user's email.
     *
     * @return The email.
     */
    public String getEmail() {
        return this.email;
    }

    /**
     * Sets the user's email.
     *
     * @param email The email.
     */
    public void setEmail(String email) {
        this.email = email;
    }

    /**
     * Gets the user's signature.
     *
     * @return The signature.
     */
    public String getSignature() {
        return this.signature;
    }

    /**
     * Sets the user's signature.
     *
     * @param signature The signature.
     */
    public void setSignature(String signature) {
        this.signature = signature;
    }

    /**
     * Tests if this UserSettings is logically equivalent to another object.
     *
     * @param object The object to test equality with.
     *
     * @return true if ths two objects are equal; false otherwise.
     */
    public boolean equals(Object object) {
        if (object instanceof UserSettings) {
            UserSettings settings = (UserSettings)object;

            return ((getName().equals(settings.getName())) &&
                   (getEmail().equals(settings.getEmail())) &&
                   (getSignature().equals(settings.getSignature())));
        }

        return false;
    }

    /**
     * Transposes (copies over) the values of another object of the same
     * type onto this one. If the object is not the same type as the
     * invoked object, it will be ignored.
     *
     * @param object The object to transpose over this one.
     */
    public void transpose(Object object) {
        if (object instanceof UserSettings) {
            UserSettings settings = (UserSettings)object;

            this.name = settings.name;
            this.email = settings.email;
            this.signature = settings.signature;
        }
    }

    /**
     * Gets a String representation of this UserSettings.
     *
     * @return A String representation.
     */
    public String toString() {
        return "UserSettings[name=" + getName() + ", email=" + getEmail() +
               ", signature=\n" + getSignature() + "\n]";
    }
}
