/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.model;

import java.util.Date;

import nan.exception.XMLParseException;

import nan.tools.EventLogger;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Node;

/**
 * A static helper method class for converting from XML to Java objects.
 *
 * @author John David Ratliff
 * @version 1.21, 1/4/04
 */
public class XMLFactory {
    /**
     * Creates a new XMLFactory object.
     */
    private XMLFactory() {
        // helper method class. no instantation please.
    }

    /**
     * Parses the SSH components of the ServerSettings Document.
     *
     * @param root The root SSH element.
     * @param settings The ServerSettings.
     */
    private static void parseSSHElements(Element root,
                                         ServerSettings settings) {
        for (int i = 0; i < root.nodeCount(); i++) {
            Node node = root.node(i);

            if (node instanceof Element) {
                Element element = (Element)node;
                String tag = element.getName();

                try {
                    if (tag.equals(ServerSettings.XML_SSH_SERVER)) {
                        settings.setSSHServer(element.getText());
                    } else if (tag.equals(ServerSettings.XML_SSH_PORT)) {
                        int temp = Integer.parseInt(element.getText());
                        settings.setSSHPort(temp);
                    } else if (tag.equals(ServerSettings.XML_SSH_USERNAME)) {
                        settings.setUsername(element.getText());
                    } else if (tag.equals(ServerSettings.XML_USE_TUNNEL)) {
                        Element tunnel = element;

                        for (int j = 0; j < tunnel.nodeCount(); j++) {
                            node = tunnel.node(j);

                            if (node instanceof Element) {
                                element = (Element)node;
                                tag = element.getName();

                                if (tag.equals(ServerSettings.XML_LOCAL_PORT)) {
                                    int temp = Integer.parseInt(element.getText());
                                    settings.setUseSSH(true);
                                    settings.setLocalPort(temp);
                                } else {
                                    EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                                         tag +
                                                                         " seen in XMLFactory.parseSSHElements()...");
                                }
                            }
                        }
                    } else if (tag.equals(ServerSettings.XML_USE_REMOTE_SAVE)) {
                        Element remoteSave = element;

                        for (int j = 0; j < remoteSave.nodeCount(); j++) {
                            node = remoteSave.node(j);

                            if (node instanceof Element) {
                                element = (Element)node;
                                tag = element.getName();

                                if (tag.equals(ServerSettings.XML_DIRECTORY)) {
                                    settings.setUseSSH(true);
                                    settings.setDirectory(element.getText());
                                } else {
                                    EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                                         tag +
                                                                         " seen in XMLFactory.parseSSHElements()...");
                                }
                            }
                        }
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in XMLFactory.parseSSHElements()...");
                    }
                } catch (NumberFormatException e) {
                    EventLogger.getInstance().logError("NumberFormatException during " +
                                                       "XMLFactory.parseSSHElements()...");
                }
            }
        }
    }

    /**
     * Recreates a ServerSettings object from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The ServerSettings object.
     *
     * @throws XMLParseException if the Document is not in proper form.
     */
    public static ServerSettings getServerSettings(Document document)
                                            throws XMLParseException {
        Element root = document.getRootElement();
        ServerSettings settings = new ServerSettings();

        if (root.getName().equals(ServerSettings.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(ServerSettings.XML_NEWS_SERVER)) {
                        settings.setNewsServer(element.getText());
                    } else if (tag.equals(ServerSettings.XML_NEWS_PORT)) {
                        try {
                            settings.setNewsPort(Integer.parseInt(element.getText()));
                        } catch (NumberFormatException e) {
                            EventLogger.getInstance().logError("NumberFormatException during " +
                                                               "XMLFactory.getServerSettings()...");
                        }
                    } else if (tag.equals(ServerSettings.XML_USE_SSH)) {
                        parseSSHElements(element, settings);
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getServerSettings()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.geetServerSettings()...");

            throw new XMLParseException();
        }

        return settings;
    }

    /**
     * Recreates a UserSettings object from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The UserSettings object.
     *
     * @throws XMLParseException if the XML Document is not a UserSettings
     *         XML Document.
     */
    public static UserSettings getUserSettings(Document document)
                                        throws XMLParseException {
        Element root = document.getRootElement();
        UserSettings settings = new UserSettings();

        if (root.getName().equals(UserSettings.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(UserSettings.XML_NAME)) {
                        settings.setName(element.getText());
                    } else if (tag.equals(UserSettings.XML_EMAIL)) {
                        settings.setEmail(element.getText());
                    } else if (tag.equals(UserSettings.XML_SIGNATURE)) {
                        settings.setSignature(element.getText());
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getUserSettings()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getUserSettings()...");

            throw new XMLParseException();
        }

        return settings;
    }

    /**
     * Recreates a Newsgroup from an XML Document.
     *
     * @param root The root element.
     *
     * @return The Newsgroup.
     *
     * @throws XMLParseException if the XML Document is not a Newsgroup XML
     *         Document.
     */
    private static Newsgroup getNewsgroup(Element root)
                                   throws XMLParseException {
        Newsgroup newsgroup = new Newsgroup();

        if (root.getName().equals(Newsgroup.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(Newsgroup.XML_NAME)) {
                        newsgroup.setName(element.getText());
                    } else if (tag.equals(Newsgroup.XML_FIRST_ARTICLE)) {
                        newsgroup.setFirstArticle(Integer.parseInt(element.getText()));
                    } else if (tag.equals(Newsgroup.XML_LAST_ARTICLE)) {
                        newsgroup.setLastArticle(Integer.parseInt(element.getText()));
                    } else if (tag.equals(Newsgroup.XML_NEXT_ARTICLE)) {
                        newsgroup.setNextArticle(Integer.parseInt(element.getText()));
                    } else if (tag.equals(Newsgroup.XML_POSTING_ALLOWED)) {
                        boolean temp = Boolean.valueOf(element.getText())
                                              .booleanValue();
                        newsgroup.setPostingAllowed(temp);
                    } else if (tag.equals(Newsgroup.XML_UPDATED)) {
                        long temp = Long.parseLong(element.getText());
                        newsgroup.setUpdated(new Date(temp));
                    } else if (tag.equals(Newsgroup.XML_MESSAGES)) {
                        newsgroup.setMessages(Integer.parseInt(element.getText()));
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getNewsgroup()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getNewsgroup()...");

            throw new XMLParseException();
        }

        return newsgroup;
    }

    /**
     * Recreates a Newsgroup from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The Newsgroup.
     *
     * @throws XMLParseException if the XML Document is not a Newsgroup XML
     *         Document.
     */
    public static Newsgroup getNewsgroup(Document document)
                                  throws XMLParseException {
        return getNewsgroup(document.getRootElement());
    }

    /**
     * Recreates a NewsgroupList from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The NewsgroupList.
     *
     * @throws XMLParseException if the XML Document is not a NewsgroupList
     *         XML Document.
     */
    public static NewsgroupList getNewsgroupList(Document document)
                                          throws XMLParseException {
        Element root = document.getRootElement();
        NewsgroupList list = new NewsgroupList();

        if (root.getName().equals(NewsgroupList.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(NewsgroupList.XML_UPDATED)) {
                        Date date = new Date(Long.parseLong(element.getText()));
                        list.setUpdated(date);
                    } else if (tag.equals(Newsgroup.XML_TAG)) {
                        Newsgroup newsgroup = getNewsgroup(element);
                        list.add(newsgroup);
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getNewsgroupList()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getNewsgroupList()...");

            throw new XMLParseException();
        }

        return list;
    }

    /**
     * Recreates a MessageHeader from an XML Document.
     *
     * @param root The root element.
     *
     * @return The MessageHeader.
     *
     * @throws XMLParseException if the XML Document is not a MessageHeader
     *         XML Document.
     */
    private static MessageHeader getMessageHeader(Element root)
                                           throws XMLParseException {
        MessageHeader header = new MessageHeader();

        if (root.getName().equals(MessageHeader.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(MessageHeader.XML_HEADER)) {
                        Element xmlHeader = element;

                        String name = null;
                        String value = null;

                        for (int j = 0; j < xmlHeader.nodeCount(); j++) {
                            node = xmlHeader.node(j);

                            if (node instanceof Element) {
                                element = (Element)node;
                                tag = element.getName();

                                if (tag.equals(MessageHeader.XML_NAME)) {
                                    name = element.getText();
                                } else if (tag.equals(MessageHeader.XML_VALUE)) {
                                    value = element.getText();
                                } else {
                                    EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                                         tag +
                                                                         " seen in XMLFactory.getMessageHeader()...");
                                }
                            }
                        }

                        if ((name != null) && (value != null)) {
                            header.addHeader(name, value);
                        }
                    } else if (tag.equals(MessageHeader.XML_READ)) {
                        boolean temp = Boolean.valueOf(element.getText())
                                              .booleanValue();
                        header.setRead(temp);
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getMessageHeader()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getMessageHeader()...");

            throw new XMLParseException();
        }

        return header;
    }

    /**
     * Recreates a MessageHeader from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The MessageHeader.
     *
     * @throws XMLParseException if the XML Document is not a MessageHeader
     *         XML Document.
     */
    public static MessageHeader getMessageHeader(Document document)
                                          throws XMLParseException {
        return getMessageHeader(document.getRootElement());
    }

    /**
     * Reconstructs a Message from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The Message.
     *
     * @throws XMLParseException if the XML Document is not a Message XML
     *         Document.
     */
    public static Message getMessage(Document document)
                              throws XMLParseException {
        Element root = document.getRootElement();
        Message message = new Message();

        if (root.getName().equals(Message.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(MessageHeader.XML_TAG)) {
                        MessageHeader header = getMessageHeader(element);
                        message.setHeader(header);
                    } else if (tag.equals(Message.XML_BODY)) {
                        message.setBody(element.getText());
                    } else if (tag.equals(Attachment.XML_TAG)) {
                        Attachment attachment = getAttachment(element);
                        message.addAttachment(attachment);
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getMessage()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getMessage()...");

            throw new XMLParseException();
        }

        return message;
    }

    /**
     * Reconstructs a License from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The Message.
     *
     * @throws XMLParseException if the XML Document is not a License XML
     *         Document.
     */
    public static License getLicense(Document document)
                              throws XMLParseException {
        Element root = document.getRootElement();
        License license = new License();

        if (root.getName().equals(License.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(License.XML_NAME)) {
                        license.setName(element.getText());
                    } else if (tag.equals(License.XML_ACCEPTED)) {
                        boolean temp = Boolean.valueOf(element.getText())
                                              .booleanValue();
                        license.setAccepted(temp);
                    } else if (tag.equals(License.XML_DATE)) {
                        long temp = Long.parseLong(element.getText());
                        license.setDate(new Date(temp));
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getLicense()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getLicense()...");

            throw new XMLParseException();
        }

        return license;
    }

    /**
     * Reconstructs an Attachment from an XML Document.
     *
     * @param root The root Element.
     *
     * @return The Attachment.
     *
     * @throws XMLParseException if the XML Document is not an Attachment
     *         XML Document.
     */
    private static Attachment getAttachment(Element root)
                                     throws XMLParseException {
        Attachment attachment = new Attachment();

        if (root.getName().equals(Attachment.XML_TAG)) {
            for (int i = 0; i < root.nodeCount(); i++) {
                Node node = root.node(i);

                if (node instanceof Element) {
                    Element element = (Element)node;
                    String tag = element.getName();

                    if (tag.equals(Attachment.XML_NAME)) {
                        attachment.setName(element.getText());
                    } else if (tag.equals(Attachment.XML_DATA)) {
                        attachment.setData(element.getText());
                    } else {
                        EventLogger.getInstance().logWarning("Unknown XML TAG " +
                                                             tag +
                                                             " seen in " +
                                                             "XMLFactory.getAttachment()...");
                    }
                }
            }
        } else {
            EventLogger.getInstance().logError("Unable to parse XML Document in " +
                                               "XMLFactory.getAttachment()...");

            throw new XMLParseException();
        }

        return attachment;
    }

    /**
     * Reconstructs an Attachment from an XML Document.
     *
     * @param document The XML Document.
     *
     * @return The Attachment.
     *
     * @throws XMLParseException if the XML Document is not an Attachment
     *         XML Document.
     */
    public static Attachment getAttachment(Document document)
                                    throws XMLParseException {
        return getAttachment(document.getRootElement());
    }
}
