/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import nan.exception.nntp.NNTPConnectException;

import nan.exception.ssh.SSHConnectException;

import nan.model.ServerSettings;

import nan.nntp.NNTPClient;

import nan.ssh.SSHConnection;

import nan.view.WaitDialog;

/**
 * A background thread to handle client connections.
 *
 * @author John David Ratliff
 * @version 1.5, 1/4/04
 */
public class BackgroundConnectThread extends AbstractBackgroundTask {
    private ServerSettings settings;
    private NNTPClient client;
    private WaitDialog dialog;

    /**
     * Creates a new BackgroundConnectThread object.
     *
     * @param settings The ServerSettings.
     * @param client The NNTP client.
     * @param dialog The WaitDialog.
     */
    public BackgroundConnectThread(ServerSettings settings,
                                   NNTPClient client, WaitDialog dialog) {
        this.settings = settings;
        this.client = client;
        this.dialog = dialog;
    }

    /**
     * The connect task.
     */
    public void run() {
        String newsHostname;
        int newsPort;

        try {
            if (settings.useSSH()) {
                // open the SSH connection
                newsHostname = "localhost";
                newsPort = settings.getLocalPort();

                try {
                    dialog.setText("Opening SSH connection...");
                    SSHConnection.getInstance().connect(settings);
                    dialog.setText("Opening SSH tunnel...");

                    if (settings.useTunnel()) {
                        boolean opened = SSHConnection.getInstance()
                                                      .openTunnel(settings);

                        if (!opened) {
                            newsHostname = null;
                        }
                    } else {
                        newsHostname = settings.getNewsServer();
                        newsPort = settings.getNewsPort();
                    }
                } catch (SSHConnectException e) {
                    newsHostname = null;
                }
            } else {
                newsHostname = settings.getNewsServer();
                newsPort = settings.getNewsPort();
            }

            if (newsHostname != null) {
                // open the NNTP connection
                dialog.setText("Opening connection to NNTP server...");

                try {
                    this.client.connect(newsHostname, newsPort);
                } catch (NNTPConnectException e) {
                    EventLogger.getInstance().logError("NNTPConnectException while connecting " +
                                                       "to NNTP server at " +
                                                       newsHostname + ":" +
                                                       newsPort +
                                                       " in BackgroundConnect.run()...");
                }
            }
        } catch (Exception e) {
            // cannot let an Exception crash the program
            EventLogger.getInstance().logError(e.toString() +
                                               " during BackgroundConnectThread.run()...");
        }

        // make sure we don't leave before the dialog is displayed
        // that could halt the program...
        while (!dialog.isVisible()) {
            try {
                Thread.sleep(50);
            } catch (InterruptedException e) {
                // not a problem...
            }
        }

        finished = true;
        dialog.setVisible(false);
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundConnectThread[server settings=" + settings +
               ", nntp client=" + client + ", wait dialog=" + dialog + "]";
    }
}
