/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;

import java.lang.reflect.Method;

import nan.exception.XMLParseException;

import nan.model.Transposer;
import nan.model.XMLFactory;

import nan.tools.EventLogger;

import nan.view.WaitDialog;

import org.dom4j.Document;

/**
 * Background task to read and parse and XML Document from cache.
 *
 * @author John David Ratliff
 * @version 1.12, 1/6/04
 */
public class BackgroundReader extends AbstractBackgroundTask {
    private volatile WaitDialog dialog;
    private Method method;
    private File file;
    private Transposer transposer;

    /**
     * Creates a new BackgroundReader object.
     *
     * @param transposer The object that will be tranposed with the parsed
     *        one.
     * @param file The file to read.
     * @param method The method that will parse it (must come from
     *        nan.model.XMLFactory).
     */
    public BackgroundReader(Transposer transposer, File file, Method method) {
        this.transposer = transposer;
        this.file = file;
        this.method = method;
    }

    /**
     * Sets the wait dialog for this task.
     *
     * @param dialog The wait dialog.
     */
    public void setWaitDialog(WaitDialog dialog) {
        this.dialog = dialog;
    }

    /**
     * Runs this task.
     */
    public void run() {
        try {
            Document document = XMLSerializer.readDocument(file);

            Object[] arguments = {document};
            Object object = method.invoke(XMLFactory.class, arguments);
            transposer.transpose(object);
        } catch (XMLParseException e) {
            // most likely a 404 error
            // do not concern yourself with this too much...
        } catch (Exception e) {
            EventLogger.getInstance().logError(e.toString() +
                                               " during BackgroundReader.run()...");
        }

        if (dialog != null) {
            while (!dialog.isVisible()) {
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                    // this is not a problem...
                }
            }

            dialog.setVisible(false);
        }

        finished = true;
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundReader[file=" + file + ", method=" + method +
               ", transposer=" + transposer + ", wait dialog=" + dialog +
               "]";
    }
}
