/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import java.text.SimpleDateFormat;

import java.util.Date;

import nan.NaN;

import nan.exception.NotReadyException;

/**
 * Logs messages, errors, and warnings as the program runs.
 *
 * @author John David Ratliff
 * @version 1.10, 12/23/03
 */
public class EventLogger {
    private boolean ready;
    private String filename = "";
    private PrintWriter writer;
    private static final EventLogger SINGLETON = new EventLogger();
    private static final SimpleDateFormat DATE_TIME = new SimpleDateFormat("yyyy.MM.dd hh:mm:ss aa");
    private static final SimpleDateFormat TIME = new SimpleDateFormat("hh:mm:ss aa");
    private static final SimpleDateFormat LOG_NAME = new SimpleDateFormat("yyyy-MM-dd");

    /**
     * The marker for messages.
     */
    public static final String MESSAGE = "(message)";

    /**
     * The marker for errors.
     */
    public static final String ERROR = "(error)";

    /**
     * The marker for warnings.
     */
    public static final String WARNING = "(warning)";

    /**
     * The separator String.
     */
    public static final String SEPARATOR =
        "----------------------------------------" +
        "----------------------------------------";

    /**
     * Creates a new EventLogger object.
     */
    private EventLogger() {
        // singleton. just one baby, just one.
    }

    /**
     * Gets the singleton instance of the EventLogger.
     *
     * @return The singleton EventLogger.
     */
    public static final EventLogger getInstance() {
        return SINGLETON;
    }

    /**
     * Gets the name of the log file for the current day.
     *
     * @return The filename for the log file.
     */
    public static String getLogFileName() {
        return LOG_NAME.format(new Date()) + ".txt";
    }

    /**
     * Formats a String based on the current time and date.
     *
     * @return A formatted String of the current date/time.
     */
    private static String getDateTimeString() {
        return DATE_TIME.format(new Date());
    }

    /**
     * Formats a String based on the current time.
     *
     * @return A formatted String of the current time.
     */
    private static String getTimeString() {
        return TIME.format(new Date());
    }

    /**
     * Prints a String to the log file.
     *
     * @param string The string to print.
     */
    private void print(String string) {
        this.writer.println(string);
        this.writer.flush();
    }

    /**
     * Prints a centered String to the log file.
     *
     * @param string The string to print.
     */
    private void printCenter(String string) {
        StringBuffer buffer = new StringBuffer(80);
        int x = (SEPARATOR.length() - string.length()) / 2;

        for (int i = 0; i < x; i++) {
            buffer.append(' ');
        }

        buffer.append(string);
        print(buffer.toString());
    }

    /**
     * Prints the log open header.
     */
    private void printOpenHeader() {
        print(SEPARATOR);
        printCenter(NaN.APP_NAME + ' ' + NaN.APP_VERSION);
        printCenter("Event Log Opened");
        printCenter(getDateTimeString());
        print(SEPARATOR);
    }

    /**
     * Prints the log closed header.
     */
    private void printCloseHeader() {
        print("");
        print("");
        print(SEPARATOR);
        printCenter(NaN.APP_NAME + ' ' + NaN.APP_VERSION);
        printCenter("Event Log Closed");
        printCenter(getDateTimeString());
        print(SEPARATOR);
        print("");
    }

    /**
     * Asks if the logger is ready.
     *
     * @return true if the logger is ready; false otherwise.
     */
    public boolean isReady() {
        return this.ready;
    }

    /**
     * Opens the EventLogger on a file.
     *
     * @param file The name file to post events to.
     */
    public void open(File file) {
        this.filename = file.getName();

        if (this.isReady()) {
            this.close();
        }

        try {
            if (!file.exists()) {
                file.createNewFile();
            }

            this.writer = new PrintWriter(new FileWriter(file, true));
            this.ready = true;

            this.printOpenHeader();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Closes the event logger.
     */
    public void close() {
        if (this.isReady()) {
            this.printCloseHeader();

            this.writer.close();

            this.ready = false;
        }
    }

    /**
     * Logs a normal message.
     *
     * @param message The message.
     *
     * @throws NotReadyException if this logger has not been initialized.
     */
    public synchronized void logMessage(String message)
                                 throws NotReadyException {
        if (!this.isReady()) {
            throw new NotReadyException();
        }

        String string = getTimeString() + ": " + MESSAGE + ' ' + message;
        print(string);
    }

    /**
     * Logs an error message.
     *
     * @param error The error message.
     *
     * @throws NotReadyException if this logger has not been initialized.
     */
    public synchronized void logError(String error) throws NotReadyException {
        if (!this.isReady()) {
            throw new NotReadyException();
        }

        String string = getTimeString() + ": " + ERROR + ' ' + error;
        print(string);
    }

    /**
     * Logs a warning.
     *
     * @param warning The warning message.
     *
     * @throws NotReadyException if this logger has not been initialized.
     */
    public synchronized void logWarning(String warning)
                                 throws NotReadyException {
        if (!this.isReady()) {
            throw new NotReadyException();
        }

        String string = getTimeString() + ": " + WARNING + ' ' + warning;
        print(string);
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "EventLogger[ready=" + isReady() + ", filename=" +
               this.filename + "]";
    }
}
