/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;

import nan.exception.BadDateFormatException;

/**
 * A helper class for static date formatting methods as required to
 * implement the RFC 850 news message standard.
 *
 * @author John David Ratliff
 * @version 1.5, 12/28/03
 */
public class NNTPDateFormat {
    /**
     * The potential date formats that can be parsed.
     */
    public static final SimpleDateFormat[] FORMATS = buildDateFormats();

    /**
     * The format for 'since when' date-times like NEWNEWS and NEWGROUPS
     * NNTP commands.
     */
    public static final SimpleDateFormat SINCE_FORMAT = new SimpleDateFormat("yyMMdd HHmmss");

    /**
     * The format for displaying a date and time.
     */
    public static final SimpleDateFormat DISPLAY_FORMAT = new SimpleDateFormat("MMM d, yyyy h:mm aa");

    /**
     * Creates a new NNTPDateFormatter object.
     */
    private NNTPDateFormat() {
        // disallow outside creation
        // we are a helper class for static methods
    }

    /**
     * Generates an array of potential NNTP date formats.
     *
     * @return The array of potential NNTP date formats.
     */
    private static SimpleDateFormat[] buildDateFormats() {
        SimpleDateFormat[] formats = new SimpleDateFormat[2];

        formats[0] = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z");
        formats[1] = new SimpleDateFormat("dd MMM yyyy HH:mm:ss Z");

        return formats;
    }

    /**
     * Parses a Date object into a 'since when' date-time String as required
     * by commands like NEWNEWS and NEWGROUPS.
     *
     * @param date The Date object to format.
     *
     * @return The parsed date String.
     */
    public static String toSinceDateString(Date date) {
        return SINCE_FORMAT.format(date);
    }

    /**
     * Parses a Date object into a display date-time String.
     *
     * @param date The Date to format.
     *
     * @return The formatted String.
     */
    public static String toDisplayString(Date date) {
        return DISPLAY_FORMAT.format(date);
    }

    /**
     * Parses an NNTP date string into a Date object.
     *
     * @param text The date string to parse.
     *
     * @return The Date object encapsulating this date string.
     *
     * @throws BadDateFormatException if the date string is in an unknown or
     *         unparsable format.
     */
    public static Date parseDate(String text) throws BadDateFormatException {
        Date date = null;

        // for each potential date format
        for (int i = 0; i < FORMATS.length; i++) {
            SimpleDateFormat format = FORMATS[i];

            try {
                // try to parse the date
                date = format.parse(text);
            } catch (ParseException e) {
                // this format didn't work
            }

            if (date != null) {
                // success. end the loop
                i = FORMATS.length;
            }
        }

        if (date == null) {
            // date remains unparsed.
            throw new BadDateFormatException();
        }

        // otherwise, return the parsed Date
        return date;
    }
}
