/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import javax.swing.border.LineBorder;

import nan.NaN;

import nan.model.Newsgroup;
import nan.model.NewsgroupList;

import nan.nntp.NNTPClient;
import nan.nntp.NNTPClientImpl;

import nan.tools.BackgroundCacheWriter;
import nan.tools.BackgroundDirectoryDeleter;
import nan.tools.BackgroundNewsgroupListDownloader;
import nan.tools.BackgroundWriter;
import nan.tools.EventLogger;
import nan.tools.XMLSerializer;

/**
 * A dialog for changing newsgroup subscriptions.
 *
 * @author John David Ratliff
 * @version 1.24, 1/2/04
 */
public class NewsgroupDialog extends JDialog implements ActionListener,
                                                        MouseListener {
    private JList available;
    private JLabel availableLabel;
    private JScrollPane availableScroller;
    private JPanel buttonPanel;
    private JButton refreshButton;
    private JButton addButton;
    private JList subscribed;
    private JLabel subscribedLabel;
    private JScrollPane subscribedScroller;
    private JButton removeButton;
    private JButton filterButton;
    private JTextField search;
    private JPanel searchPanel;
    private View parent;
    private NewsgroupList availableList;
    private NewsgroupList subscribedList;
    private NewsgroupList current;
    private boolean availableModified;
    private boolean subscribedModified;
    private static final String ADD_ACTION = "_add";
    private static final String REMOVE_ACTION = "_remove";
    private static final String REFRESH_ACTION = "_refresh";
    private static final String FILTER_ACTION = "_filter";

    /**
     * Creates a new NewsgroupDialog object.
     *
     * @param parent The parent of this dialog.
     */
    public NewsgroupDialog(View parent) {
        super(parent, true);

        this.parent = parent;
        initComponents();
    }

    /**
     * Initializes the dialog components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        availableLabel = new JLabel();
        subscribedLabel = new JLabel();
        available = new JList();
        subscribed = new JList();
        availableScroller = new JScrollPane();
        subscribedScroller = new JScrollPane();
        refreshButton = new JButton();
        buttonPanel = new JPanel();
        addButton = new JButton();
        removeButton = new JButton();
        searchPanel = new JPanel();
        search = new JTextField();
        filterButton = new JButton();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Newsgroup Selection Dialog");
        addWindowListener(new WindowAdapter() {
                /**
                 * Called when the window is closing.
                 *
                 * @param event The associated WindowEvent.
                 */
                public void windowClosing(WindowEvent event) {
                    closeAction();
                }

                /**
                 * Called when the window is opened.
                 *
                 * @param event The associated WindowEvent.
                 */
                public void windowOpened(WindowEvent event) {
                    openAction();
                }
            });

        availableLabel.setText("Available Newsgroups");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        getContentPane().add(availableLabel, gridBagConstraints);

        subscribedLabel.setText("Subscribed Newsgroups");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        getContentPane().add(subscribedLabel, gridBagConstraints);

        available.setBorder(new LineBorder(new Color(0, 0, 0)));
        available.addMouseListener(this);
        availableScroller.setViewportView(available);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 0.45;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(availableScroller, gridBagConstraints);

        subscribed.setBorder(new LineBorder(new Color(0, 0, 0)));
        subscribed.addMouseListener(this);
        subscribedScroller.setViewportView(subscribed);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 0.45;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(subscribedScroller, gridBagConstraints);

        refreshButton.setText("Refresh");
        refreshButton.setActionCommand(REFRESH_ACTION);
        refreshButton.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new Insets(0, 0, 5, 0);
        getContentPane().add(refreshButton, gridBagConstraints);

        buttonPanel.setLayout(new GridBagLayout());

        addButton.setText("Add");
        addButton.setActionCommand(ADD_ACTION);
        addButton.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        buttonPanel.add(addButton, gridBagConstraints);

        removeButton.setText("Remove");
        removeButton.setActionCommand(REMOVE_ACTION);
        removeButton.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        buttonPanel.add(removeButton, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        getContentPane().add(buttonPanel, gridBagConstraints);

        searchPanel.setLayout(new GridBagLayout());

        search.setActionCommand(FILTER_ACTION);
        search.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.5;
        searchPanel.add(search, gridBagConstraints);

        filterButton.setText("Filter");
        filterButton.setActionCommand(FILTER_ACTION);
        filterButton.addActionListener(this);
        searchPanel.add(filterButton, new GridBagConstraints());

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 5, 5);
        getContentPane().add(searchPanel, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 550) / 2,
                  (screenSize.height - 550) / 2, 550, 550);
    }

    /**
     * Called when this dialog is closed.
     */
    private void closeAction() {
        this.setVisible(false);
    }

    /**
     * Called when this dialog is opened.
     */
    private void openAction() {
        if (availableList.getSize() == 0) {
            String message = "Would you like to download the list of available Newsgroups?";
            String title = "Download Newsgroup List?";

            int choice = JOptionPane.showConfirmDialog(this, message, title,
                                                       JOptionPane.YES_NO_OPTION,
                                                       JOptionPane.QUESTION_MESSAGE);

            if (choice == JOptionPane.YES_OPTION) {
                this.refreshAction();
            }
        }
    }

    /**
     * Called when the add button is pressed.
     */
    private void addAction() {
        int[] indicies = this.available.getSelectedIndices();

        for (int i = 0; i < indicies.length; i++) {
            Newsgroup newsgroup = this.current.get(indicies[i]);
            this.subscribedList.add(newsgroup);
        }

        if (indicies.length > 0) {
            this.subscribedModified = true;
        }
    }

    /**
     * Called when the remove button is pressed.
     */
    private void removeAction() {
        int[] indicies = subscribed.getSelectedIndices();

        for (int i = (indicies.length - 1); i >= 0; i--) {
            Newsgroup newsgroup = subscribedList.remove(indicies[i]);

            File cache = BackgroundCacheWriter.getCacheDirectory(newsgroup);
            BackgroundDirectoryDeleter thread = new BackgroundDirectoryDeleter(cache);
            thread.start();

            parent.addBackgroundThread(thread);
        }

        File file = XMLSerializer.getFilePath(NaN.SETTINGS_DIRECTORY,
                                              NaN.SUBSCRIBED_NEWSGROUPS_FILE);
        BackgroundWriter writer = new BackgroundWriter(subscribedList, file);
        writer.start();

        parent.addBackgroundThread(writer);

        if (indicies.length > 0) {
            this.subscribedModified = true;
        }
    }

    /**
     * Called when the refresh button is pressed.
     */
    private void refreshAction() {
        NNTPClient client = this.parent.getNNTPClient();

        if (client.isConnected()) {
            WaitDialog dialog = new WaitDialog(this.parent);
            NNTPClientImpl impl = (NNTPClientImpl)client;
            impl.setWaitDialog(dialog);

            BackgroundNewsgroupListDownloader reader = new BackgroundNewsgroupListDownloader(dialog,
                                                                                             parent.getNNTPClient());
            reader.start();
            parent.addBackgroundThread(reader);

            while (!reader.isFinished()) {
                dialog.display("Reading from NNTP server", "Please wait...");
            }

            dialog.dispose();
            impl.setWaitDialog(null);

            // once we're here, the task should be finished...
            NewsgroupList list = reader.getNewsgroupList();
            this.availableList.transpose(list);
            this.available.setModel(list.getListModel());

            this.availableModified = true;
        }
    }

    /**
     * Called when the filter button is pressed.
     */
    private void filterAction() {
        String filter = search.getText().toLowerCase();

        if (filter.length() > 0) {
            NewsgroupList temp = new NewsgroupList();

            for (int i = 0; i < availableList.getSize(); i++) {
                Newsgroup group = availableList.get(i);
                String compare = group.getName().toLowerCase();

                if (compare.startsWith(filter)) {
                    temp.add(group);
                }
            }

            available.setModel(temp.getListModel());
            current = temp;
        } else {
            available.setModel(availableList.getListModel());
            current = availableList;
        }
    }

    /**
     * Asks if the available list was modified.
     *
     * @return true if the available list was modified; false otherwise.
     */
    public boolean wasAvailableModified() {
        return this.availableModified;
    }

    /**
     * Asks if the subscribed list was modified.
     *
     * @return true if this subscribed list was modified; false otherwise.
     */
    public boolean wasSubscribedModified() {
        return this.subscribedModified;
    }

    /**
     * Displays this dialog.
     *
     * @param available The available newsgroups.
     * @param subscribed The subscribed newsgroups.
     */
    public void display(NewsgroupList available, NewsgroupList subscribed) {
        this.availableModified = false;
        this.subscribedModified = false;

        this.current = available;
        this.availableList = available;
        this.subscribedList = subscribed;

        this.available.setModel(available.getListModel());
        this.subscribed.setModel(subscribed.getListModel());

        this.setVisible(true);
    }

    /**
     * The registered callback for actions on this dialog.
     *
     * @param event The associated event which triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(ADD_ACTION)) {
            addAction();
        } else if (action.equals(REMOVE_ACTION)) {
            removeAction();
        } else if (action.equals(REFRESH_ACTION)) {
            refreshAction();
        } else if (action.equals(FILTER_ACTION)) {
            filterAction();
        } else {
            EventLogger.getInstance().logError("Invalid action command " +
                                               action +
                                               " in NewsgroupDialog.actionPerformed()...");
        }
    }

    /**
     * Called when the mouse is clicked.
     *
     * @param event The associated MouseEvent.
     */
    public void mouseClicked(MouseEvent event) {
        if (event.getClickCount() == 2) {
            // double-click
            JList list = (JList)event.getSource();

            if (list == this.available) {
                this.addAction();
            } else if (list == this.subscribed) {
                this.removeAction();
            }
        }
    }

    /**
     * Called when the mouse is released.
     *
     * @param event The associated MouseEvent.
     */
    public void mouseReleased(MouseEvent event) {
    }

    /**
     * Called when the mouse is pressed (clicked and released).
     *
     * @param event The associated MouseEvent.
     */
    public void mousePressed(MouseEvent event) {
    }

    /**
     * Called when the mouse enters the area.
     *
     * @param event The associated MouseEvent.
     */
    public void mouseEntered(MouseEvent event) {
    }

    /**
     * Called when the mouse exits the area.
     *
     * @param event The associated MouseEvent.
     */
    public void mouseExited(MouseEvent event) {
    }

    /**
     * Returns a String representation of this NewsgroupDialog.
     *
     * @return A String representation.
     */
    public String toString() {
        return "NewsgroupDialog[available=" + available + ", subscribed=" +
               subscribed + ", add button=" + addButton +
               ", remove button=" + removeButton + ", search=" + search +
               ", filter button=" + filterButton + ", refresh button=" +
               refreshButton + "]";
    }
}
