/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.WindowConstants;

import nan.model.ServerSettings;

import nan.tools.EventLogger;

/**
 * A dialog to prompt for SSH passwords.
 *
 * @author John David Ratliff
 * @version 1.2, 1/6/04
 */
public class PasswordDialog extends JDialog implements ActionListener,
                                                       KeyListener {
    private ServerSettings settings;
    private JButton okButton;
    private JPasswordField password;
    private JLabel promptLabel;
    private static final String OK_ACTION = "_ok";

    /**
     * Creates a new PasswordDialog object.
     *
     * @param parent The parent of this dialog.
     * @param settings The ServerSettings to store the password in.
     */
    public PasswordDialog(Frame parent, ServerSettings settings) {
        super(parent, true);

        this.settings = settings;
        initComponents();
    }

    /**
     * Initializes the dialog components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        promptLabel = new JLabel();
        password = new JPasswordField();
        okButton = new JButton();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Enter SSH Password");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(promptLabel, gridBagConstraints);

        password.addKeyListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(password, gridBagConstraints);

        okButton.setText("Okay");
        okButton.setActionCommand(OK_ACTION);
        okButton.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        getContentPane().add(okButton, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 375) / 2,
                  (screenSize.height - 125) / 2, 375, 125);
    }

    /**
     * Displays this dialog.
     */
    public void display() {
        String message = "Enter password for " + settings.getUsername() +
                         "@" + settings.getSSHServer() + ":" +
                         settings.getSSHPort();
        promptLabel.setText(message);

        setVisible(true);
    }

    /**
     * Called when the OK button is pressed.
     */
    private void okAction() {
        char[] password = this.password.getPassword();

        if (password.length > 0) {
            settings.setPassword(new String(password));
            setVisible(false);

            // for security...
            for (int i = 0; i < password.length; i++) {
                password[i] = (char)0;
            }
        }
    }

    /**
     * The registered callback for ActionEvents on this dialog.
     *
     * @param event The event which triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(OK_ACTION)) {
            okAction();
        } else {
            EventLogger.getInstance().logError("Unknown action " + action +
                                               " in PasswordDialog.actionPerformed()...");
        }
    }

    /**
     * Called when a key is pressed on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void keyPressed(KeyEvent event) {
        if (event.getKeyCode() == KeyEvent.VK_ENTER) {
            okAction();
        }
    }

    /**
     * Called when a key is released on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void keyReleased(KeyEvent event) {
    }

    /**
     * Called when a key is typed on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void keyTyped(KeyEvent event) {
    }
}
