/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import javax.swing.text.JTextComponent;

import nan.exception.InvalidPortException;

import nan.model.ServerSettings;

import nan.tools.EventLogger;

/**
 * A dialog for changing the server settings.
 *
 * @author John David Ratliff
 * @version 1.15, 1/6/04
 */
public class ServerSettingsDialog extends JDialog implements ActionListener,
                                                             FocusListener {
    private JPanel buttonPanel;
    private JButton cancelButton;
    private JTextField directory;
    private JLabel directoryLabel;
    private JTextField newsPort;
    private JLabel newsPortLabel;
    private JTextField newsServer;
    private JLabel newsServerLabel;
    private JButton okButton;
    private JCheckBox remoteSaveCheckbox;
    private JSeparator separator;
    private JCheckBox sshCheckbox;
    private JPasswordField sshPassword;
    private JLabel sshPasswordLabel;
    private JTextField sshPort;
    private JLabel sshPortLabel;
    private JTextField sshServer;
    private JLabel sshServerLabel;
    private JTextField sshUsername;
    private JLabel sshUsernameLabel;
    private JCheckBox tunnelCheckbox;
    private JTextField tunnelPort;
    private JLabel tunnelPortLabel;
    private ServerSettings settings;
    private boolean modified;
    private static final String SSH_ACTION = "_ssh";
    private static final String TUNNEL_ACTION = "_tunnel";
    private static final String REMOTE_SAVE_ACTION = "_remote_save";
    private static final String OK_ACTION = "_ok";
    private static final String CANCEL_ACTION = "_cancel";

    /**
     * Creates a new ServerSettingsDialog object.
     *
     * @param parent The parent of thie dialog.
     * @param settings The ServerSettings.
     */
    public ServerSettingsDialog(Frame parent, ServerSettings settings) {
        super(parent, true);

        this.settings = settings;
        initComponents();
    }

    /**
     * Initializes the dialog components.
     */
    private void initComponents() {
        Font font = new Font("Monospaced", Font.PLAIN, 12);

        GridBagConstraints gridBagConstraints;

        newsServerLabel = new JLabel();
        newsServer = new JTextField();
        newsPortLabel = new JLabel();
        newsPort = new JTextField();
        separator = new JSeparator();
        sshCheckbox = new JCheckBox();
        sshServerLabel = new JLabel();
        sshServer = new JTextField();
        sshPortLabel = new JLabel();
        sshPort = new JTextField();
        sshUsernameLabel = new JLabel();
        sshUsername = new JTextField();
        sshPasswordLabel = new JLabel();
        sshPassword = new JPasswordField();
        tunnelCheckbox = new JCheckBox();
        tunnelPortLabel = new JLabel();
        tunnelPort = new JTextField();
        remoteSaveCheckbox = new JCheckBox();
        directoryLabel = new JLabel();
        directory = new JTextField();
        buttonPanel = new JPanel();
        okButton = new JButton();
        cancelButton = new JButton();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Server Settings");
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent event) {
                    cancelAction();
                }
            });

        newsServerLabel.setDisplayedMnemonic('n');
        newsServerLabel.setLabelFor(newsServer);
        newsServerLabel.setText("News Server: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new Insets(10, 10, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(newsServerLabel, gridBagConstraints);

        newsServer.setActionCommand(OK_ACTION);
        newsServer.addActionListener(this);
        newsServer.addFocusListener(this);
        newsServer.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(10, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(newsServer, gridBagConstraints);

        newsPortLabel.setDisplayedMnemonic('p');
        newsPortLabel.setLabelFor(newsPort);
        newsPortLabel.setText("Port: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new Insets(0, 10, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(newsPortLabel, gridBagConstraints);

        newsPort.setActionCommand(OK_ACTION);
        newsPort.addActionListener(this);
        newsPort.addFocusListener(this);
        newsPort.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(newsPort, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(10, 10, 10, 10);
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(separator, gridBagConstraints);

        sshCheckbox.setMnemonic('s');
        sshCheckbox.setText("Use SSH");
        sshCheckbox.setActionCommand(SSH_ACTION);
        sshCheckbox.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 3, 0, 0);
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(sshCheckbox, gridBagConstraints);

        sshServerLabel.setDisplayedMnemonic('h');
        sshServerLabel.setLabelFor(sshServer);
        sshServerLabel.setText("Hostname: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(sshServerLabel, gridBagConstraints);

        sshServer.setActionCommand(OK_ACTION);
        sshServer.addActionListener(this);
        sshServer.addFocusListener(this);
        sshServer.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(sshServer, gridBagConstraints);

        sshPortLabel.setDisplayedMnemonic('r');
        sshPortLabel.setLabelFor(sshPort);
        sshPortLabel.setText("Port: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(sshPortLabel, gridBagConstraints);

        sshPort.setActionCommand(OK_ACTION);
        sshPort.addActionListener(this);
        sshPort.addFocusListener(this);
        sshPort.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(sshPort, gridBagConstraints);

        sshUsernameLabel.setDisplayedMnemonic('e');
        sshUsernameLabel.setLabelFor(sshUsername);
        sshUsernameLabel.setText("Username: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(sshUsernameLabel, gridBagConstraints);

        sshUsername.setActionCommand(OK_ACTION);
        sshUsername.addActionListener(this);
        sshUsername.addFocusListener(this);
        sshUsername.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(sshUsername, gridBagConstraints);

        sshPasswordLabel.setDisplayedMnemonic('w');
        sshPasswordLabel.setLabelFor(sshPassword);
        sshPasswordLabel.setText("Password: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(sshPasswordLabel, gridBagConstraints);

        sshPassword.setActionCommand(OK_ACTION);
        sshPassword.addActionListener(this);
        sshPassword.addFocusListener(this);
        sshPassword.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(sshPassword, gridBagConstraints);

        tunnelCheckbox.setText("Use SSH Tunnel");
        tunnelCheckbox.setActionCommand(TUNNEL_ACTION);
        tunnelCheckbox.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        getContentPane().add(tunnelCheckbox, gridBagConstraints);

        tunnelPortLabel.setDisplayedMnemonic('t');
        tunnelPortLabel.setLabelFor(tunnelPort);
        tunnelPortLabel.setText("Tunnel Port: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.insets = new Insets(0, 20, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(tunnelPortLabel, gridBagConstraints);

        tunnelPort.setActionCommand(OK_ACTION);
        tunnelPort.addActionListener(this);
        tunnelPort.addFocusListener(this);
        tunnelPort.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(tunnelPort, gridBagConstraints);

        remoteSaveCheckbox.setText("Remote Save");
        remoteSaveCheckbox.setActionCommand(REMOTE_SAVE_ACTION);
        remoteSaveCheckbox.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        getContentPane().add(remoteSaveCheckbox, gridBagConstraints);

        directoryLabel.setText("Directory: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.insets = new Insets(0, 20, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        getContentPane().add(directoryLabel, gridBagConstraints);

        directory.setActionCommand(OK_ACTION);
        directory.addActionListener(this);
        directory.addFocusListener(this);
        directory.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        getContentPane().add(directory, gridBagConstraints);

        okButton.setMnemonic('o');
        okButton.setText("OK");
        okButton.setActionCommand(OK_ACTION);
        okButton.addActionListener(this);
        buttonPanel.add(okButton);

        cancelButton.setMnemonic('c');
        cancelButton.setText("CANCEL");
        cancelButton.setActionCommand(CANCEL_ACTION);
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new Insets(10, 0, 5, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(buttonPanel, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 425) / 2,
                  (screenSize.height - 350) / 2, 425, 350);
    }

    /**
     * Asks if these settings have been modified.
     *
     * @return true if the settings were changed; false otherwise.
     */
    public boolean isModified() {
        return this.modified;
    }

    /**
     * Displays this dialog with the proper server settings.
     */
    public void display() {
        modified = false;

        newsServer.setText(settings.getNewsServer());
        newsServer.setCaretPosition(0);

        newsPort.setText("" + settings.getNewsPort());
        newsPort.setCaretPosition(0);

        sshServer.setText(settings.getSSHServer());
        sshServer.setCaretPosition(0);

        sshPort.setText("" + settings.getSSHPort());
        sshPort.setCaretPosition(0);

        sshUsername.setText(settings.getUsername());
        sshUsername.setCaretPosition(0);

        sshPassword.setText(settings.getPassword());
        sshPassword.setCaretPosition(0);

        tunnelPort.setText("" + settings.getLocalPort());
        tunnelPort.setCaretPosition(0);

        directory.setText(settings.getDirectory());
        directory.setCaretPosition(0);

        if (settings.useSSH() &&
            (settings.useTunnel() || settings.useRemoteSave())) {
            sshCheckbox.setSelected(true);
            tunnelCheckbox.setSelected(settings.useTunnel());
            remoteSaveCheckbox.setSelected(settings.useRemoteSave());
        }

        // setup enabled SSH properties...
        sshAction();
        tunnelAction();
        remoteSaveAction();

        this.setVisible(true);
    }

    /**
     * Verifies the values for this dialog.
     *
     * @param settings The ServerSettings to set the values to.
     *
     * @return true if verification was a sucesss; false otherwise.
     */
    private boolean verify(ServerSettings settings) {
        String title = "Invalid Server Settings";

        try {
            settings.setNewsServer(newsServer.getText());
            settings.setNewsPort(Integer.parseInt(newsPort.getText()));

            if (sshCheckbox.isSelected()) {
                settings.setUseSSH(true);

                settings.setSSHServer(sshServer.getText());
                settings.setSSHPort(Integer.parseInt(sshPort.getText()));
                settings.setUsername(sshUsername.getText());
                settings.setPassword(new String(sshPassword.getPassword()));

                if (tunnelCheckbox.isSelected()) {
                    settings.setLocalPort(Integer.parseInt(tunnelPort.getText()));
                }

                if (remoteSaveCheckbox.isSelected()) {
                    settings.setDirectory(directory.getText());
                }
            }
        } catch (NumberFormatException e) {
            // bad number
            JOptionPane.showMessageDialog(getParent(),
                                          "Ports must be integers.", title,
                                          JOptionPane.ERROR_MESSAGE);

            return false;
        } catch (InvalidPortException e) {
            // bad port number
            JOptionPane.showMessageDialog(getParent(),
                                          "Ports must be in the range 1-65535, or " +
                                          "1024-65535 for the Tunnel Port.",
                                          title, JOptionPane.ERROR_MESSAGE);

            return false;
        }

        return true;
    }

    /**
     * Called when the SSH checkbox is toggled.
     */
    private void sshAction() {
        // enable/disable the settings
        boolean enabled = sshCheckbox.isSelected();

        sshServer.setEnabled(enabled);
        sshPort.setEnabled(enabled);
        sshUsername.setEnabled(enabled);
        sshPassword.setEnabled(enabled);

        tunnelCheckbox.setEnabled(enabled);
        tunnelAction();

        remoteSaveCheckbox.setEnabled(enabled);
        remoteSaveAction();
    }

    /**
     * Called when the SSH tunnel checkbox is toggled.
     */
    private void tunnelAction() {
        // reverse enable the tunnel port
        tunnelPort.setEnabled(tunnelCheckbox.isSelected());
    }

    /**
     * Called when the SSH remote save checkbox is toggled.
     */
    private void remoteSaveAction() {
        // reverse enable the directory
        directory.setEnabled(remoteSaveCheckbox.isSelected());
    }

    /**
     * Called when the OK button is pressed.
     */
    private void okAction() {
        ServerSettings settings = new ServerSettings();

        if (verify(settings)) {
            if (!this.settings.equals(settings)) {
                this.settings.transpose(settings);
                this.modified = true;
            }

            this.setVisible(false);
        }
    }

    /**
     * Called when CANCEL or close window is selected.
     */
    private void cancelAction() {
        this.setVisible(false);
    }

    /**
     * The registered callback for the buttons.
     *
     * @param event The event which triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(OK_ACTION)) {
            okAction();
        } else if (action.equals(CANCEL_ACTION)) {
            cancelAction();
        } else if (action.equals(SSH_ACTION)) {
            sshAction();
        } else if (action.equals(TUNNEL_ACTION)) {
            tunnelAction();
        } else if (action.equals(REMOTE_SAVE_ACTION)) {
            remoteSaveAction();
        } else {
            // error. invalid action
            EventLogger.getInstance().logError("Invalid action command " +
                                               action +
                                               " in ServerSettingsDialog...");
        }
    }

    /**
     * Called when the focus is gained on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void focusGained(FocusEvent event) {
        Object source = event.getSource();

        if (source instanceof JTextComponent) {
            JTextComponent text = (JTextComponent)source;
            text.selectAll();
        } else {
            EventLogger.getInstance().logError("Source is not a JTextComponent in " +
                                               "ServerSettingsDialog.focusGained()...");
        }
    }

    /**
     * Called when the focus is lost on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void focusLost(FocusEvent event) {
    }

    /**
     * Returns a String representation of these ServerSettings.
     *
     * @return A String representation.
     */
    public String toString() {
        return "ServerSettingsDialog[settings=" + settings + ", modified=" +
               isModified() + ", news server=" + newsServer +
               ", news port=" + newsPort + ", use ssh=" +
               sshCheckbox.isSelected() + ", ssh hostname=" + sshServer +
               ", ssh port=" + sshPort + ", ssh username=" + sshUsername +
               ", use ssh tunnel=" + tunnelCheckbox.isSelected() +
               ", tunnel port=" + tunnelPort + ", use remote save=" +
               remoteSaveCheckbox.isSelected() + ", remote save directory=" +
               directory + "]";
    }
}
