/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import javax.swing.text.JTextComponent;

import nan.model.UserSettings;

import nan.tools.EventLogger;

/**
 * A dialog to change the user settings.
 *
 * @author John David Ratliff
 * @version 1.6, 12/31/03
 */
public class UserSettingsDialog extends JDialog implements ActionListener,
                                                           FocusListener,
                                                           KeyListener {
    private UserSettings settings;
    private JPanel buttonPanel;
    private JButton cancelButton;
    private JTextField email;
    private JLabel emailLabel;
    private JTextField name;
    private JLabel nameLabel;
    private JButton okButton;
    private JSeparator separator;
    private JTextArea signature;
    private JLabel signatureLabel;
    private JScrollPane signatureScroller;
    private boolean modified;
    private static final String OK_ACTION = "_ok";
    private static final String CANCEL_ACTION = "_cancel";

    /**
     * Creates a new UserSettingsDialog object.
     *
     * @param parent The parent of this dialog.
     * @param settings The UserSettings to modify.
     */
    public UserSettingsDialog(Frame parent, UserSettings settings) {
        super(parent, true);

        this.settings = settings;
        initComponents();
    }

    /**
     * Initializes the dialog components.
     */
    private void initComponents() {
        Font font = new Font("Monospaced", Font.PLAIN, 12);

        GridBagConstraints gridBagConstraints;

        nameLabel = new JLabel();
        name = new JTextField();
        emailLabel = new JLabel();
        email = new JTextField();
        separator = new JSeparator();
        signatureLabel = new JLabel();
        buttonPanel = new JPanel();
        okButton = new JButton();
        cancelButton = new JButton();
        signatureScroller = new JScrollPane();
        signature = new JTextArea();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("User Settings");
        setFont(new Font("Alba", 0, 10));
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    cancelAction();
                }
            });

        nameLabel.setDisplayedMnemonic('n');
        nameLabel.setLabelFor(name);
        nameLabel.setText("Name: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new Insets(10, 10, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(nameLabel, gridBagConstraints);

        name.addFocusListener(this);
        name.addKeyListener(this);
        name.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(10, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(name, gridBagConstraints);

        emailLabel.setDisplayedMnemonic('e');
        emailLabel.setLabelFor(email);
        emailLabel.setText("Email: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new Insets(0, 10, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(emailLabel, gridBagConstraints);

        email.addFocusListener(this);
        email.addKeyListener(this);
        email.setFont(font);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(email, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(5, 10, 10, 10);
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(separator, gridBagConstraints);

        signatureLabel.setDisplayedMnemonic('s');
        signatureLabel.setLabelFor(signature);
        signatureLabel.setText("Signature: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new Insets(0, 10, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(signatureLabel, gridBagConstraints);

        okButton.setMnemonic('o');
        okButton.setText("OK");
        okButton.setActionCommand(OK_ACTION);
        okButton.addActionListener(this);
        buttonPanel.add(okButton);

        cancelButton.setMnemonic('c');
        cancelButton.setText("CANCEL");
        cancelButton.setActionCommand(CANCEL_ACTION);
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(10, 10, 10, 10);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 0.3;
        getContentPane().add(buttonPanel, gridBagConstraints);

        signature.addFocusListener(this);
        signature.setFont(font);
        signature.setLineWrap(true);
        signature.setBorder(null);
        signatureScroller.setViewportView(signature);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(0, 0, 0, 10);
        gridBagConstraints.weightx = 0.95;
        gridBagConstraints.weighty = 0.7;
        getContentPane().add(signatureScroller, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 400) / 2,
                  (screenSize.height - 300) / 2, 400, 300);
    }

    /**
     * Called when the CANCEL button is pressed or the window is closed.
     */
    private void cancelAction() {
        this.setVisible(false);
    }

    /**
     * Called when the OK button is pressed.
     */
    private void okAction() {
        UserSettings settings = new UserSettings();

        settings.setName(this.name.getText());
        settings.setEmail(this.email.getText());
        settings.setSignature(this.signature.getText());

        if (!this.settings.equals(settings)) {
            this.modified = true;
            this.settings.transpose(settings);
        }

        this.setVisible(false);
    }

    /**
     * Displays this dialog.
     */
    public void display() {
        this.modified = false;

        this.name.setText(this.settings.getName());
        this.name.setCaretPosition(0);
        this.email.setText(this.settings.getEmail());
        this.email.setCaretPosition(0);
        this.signature.setText(this.settings.getSignature());
        this.signature.setCaretPosition(0);

        this.setVisible(true);
    }

    /**
     * Asks if the settings were modified.
     *
     * @return true if they were changed; false otherwise.
     */
    public boolean isModified() {
        return this.modified;
    }

    /**
     * The registered callback for the buttons.
     *
     * @param event The event associated with this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(OK_ACTION)) {
            this.okAction();
        } else if (action.equals(CANCEL_ACTION)) {
            this.cancelAction();
        } else {
            // invalid action
            EventLogger.getInstance().logError("Invalid action command " +
                                               action +
                                               " in UserSettingsDialog...");
        }
    }

    /**
     * Called when the focus is gained on a registered component.
     *
     * @param event The event which trigger this callback.
     */
    public void focusGained(FocusEvent event) {
        Object source = event.getSource();

        if (source instanceof JTextComponent) {
            JTextComponent text = (JTextComponent)source;
            text.selectAll();
        } else {
            EventLogger.getInstance().logError("Source is not a JTextComponent in " +
                                               "UserSettingsDialog.focusGained()...");
        }
    }

    /**
     * Called when the focus is lost on a registered component.
     *
     * @param event The event which trigger this callback.
     */
    public void focusLost(FocusEvent event) {
    }

    /**
     * Called when a key is pressed on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void keyPressed(KeyEvent event) {
        if (event.getKeyCode() == KeyEvent.VK_ENTER) {
            this.okAction();
        }
    }

    /**
     * Called when a key is released on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void keyReleased(KeyEvent event) {
    }

    /**
     * Called when a key is typed on a registered component.
     *
     * @param event The event which triggered this callback.
     */
    public void keyTyped(KeyEvent event) {
    }

    /**
     * Returns a String representation of this UserSettings.
     *
     * @return A String representation.
     */
    public String toString() {
        return "UserSettings[name=" + this.name + ", email=" + this.email +
               ", signature=" + this.signature + "]";
    }
}
